import { useContext, useDebugValue, useEffect, useRef, useState } from 'react';
import { getStoreContext } from './store';
export function Consumer(props) {
    const store = useStore(props.of);
    return props.children(store);
}
export function useStore(S, deps) {
    const name = S.displayName || S.name;
    useDebugValue(name);
    const hasDefaultValue = S.hasOwnProperty('defaultValue');
    const Context = hasDefaultValue ? getStoreContext(S) : S.Context;
    if (!Context) {
        throw new Error(`No store context of "${name}" found. And "${name}" doesn't have defaultValue. Either render a Provider or set a defaultValue for this Store.`);
    }
    const container = useContext(Context);
    const [state, setState] = useState(container.state);
    const depsRef = useRef([]);
    useEffect(() => {
        const subscriber = () => {
            if (!deps) {
                setState(container.state);
            }
            else {
                const oldDeps = depsRef.current;
                const newDeps = deps(container.state);
                if (compare(oldDeps, newDeps)) {
                    setState(container.state);
                }
                depsRef.current = newDeps;
            }
        };
        container.subscribers.add(subscriber);
        return () => {
            container.subscribers.delete(subscriber);
        };
    }, []);
    return state;
}
function compare(oldDeps, newDeps) {
    if (oldDeps.length !== newDeps.length) {
        return true;
    }
    for (const index in newDeps) {
        if (oldDeps[index] !== newDeps[index]) {
            return true;
        }
    }
    return false;
}
